var pathUtils = require("../utils/PathUtils");
var path = require("path");
var fs = require("fs");
var vueTempCompiler = require("vue-template-compiler");
var VueComponentAnalyze = require("./VueComponentAnalyze");
var ioUtils = require("../utils/IOUtils");

var MetadataGenerator = function(components){
    this.components = components;
};

MetadataGenerator.prototype = {

    generate : function(){
        for(var i=0,l=this.components.length;i<l;i++){
            var component = this.components[i];
            var componentCode = component.getCode();
            var windows = component.getWindows();
            if(windows){
                for(var j=0,len=windows.length;j<len;j++){
                    var win = windows[j];
                    var windowCode = win.getCode();
                    var widgets = win.getWidgets();
                    if(widgets){
                        for(var k=0,length=widgets.length;k<length;k++){
                            var widget = widgets[k];
                            var widgetCode = widget.getCode();
                            var relativePath = pathUtils.getWidgetIndexVuePath(componentCode,windowCode,widgetCode);
                            var absPath = path.join(pathUtils.getPluginPath(),relativePath);
                            fs.readFile(absPath, 'utf8', (function(cmCode,wnCode,wdCode){
                                return function(err,data){
                                    var rs = vueTempCompiler.parseComponent(data);
                                    var javascript = "";
                                    var domScript = "";
                                    if(rs.script){
                                        javascript = rs.script.content;
                                    }
                                    if(rs.template){
                                        domScript = rs.template.content;
                                    }
                                    var analyzer = new VueComponentAnalyze({
                                        domScript : domScript,
                                        javascript : javascript
                                    });
                                    var savePath = pathUtils.toWidgetMetadataPath(cmCode,wnCode,wdCode);
                                    var metadata = {
                                        events : analyzer.getEvents(),
                                        tagnames : analyzer.getTagNames()
                                    };
                                    ioUtils.writeContentAsync(pathUtils.getPluginPath()+"/dist/"+savePath,JSON.stringify(metadata,null,"\t"),function(err){
                                        if(err){
                                            throw err;
                                        }
                                    });
                                };
                            })(componentCode,windowCode,widgetCode));
                        }
                    }
                }
            }
        }
    }
};

module.exports = MetadataGenerator;