var parse5 = require('parse5');
var recast = require('recast');

var VueComponentAnalyze = function(params){
    this.domScript = params.domScript;
    this.javascript = params.javascript;
    this.events = [];
    this.tagNames = [];
    this.analyzeDomScript();
    this.analyzeJavascript();
};

VueComponentAnalyze.prototype = {

    getEvents : function(){
        return this.events;
    },

    getTagNames : function(){
        return this.tagNames;
    },

    /**
     *  解析dom脚本
     */
    analyzeDomScript : function(){
        if(!this.domScript) return;
        var document = parse5.parseFragment(this.domScript);
        var children = document.childNodes;
        for(var i=0,l=children.length;i<l;i++){
            var child = children[i];
            this.analyzeDomNode(child);
        }
    },

    addEvent : function(event){
        var notFound = true;
        for(var i=0,l=this.events.length;i<l;i++){
            var evt = this.events[i];
            if(evt.eventCode==event.eventCode){
                notFound = false;
                if(evt.argumentNum < event.argumentNum){
                    evt.argumentNum = event.argumentNum;
                }
            }
        }
        if(notFound){
            this.events.push(event);
        }
    },

    analyzeDomNode : function(node){
        /* 如果是文本则直接返回 */
        if(node.nodeName=="#text") return;
        var tagName = node.tagName;
        if(this.tagNames.indexOf(tagName)==-1){
            this.tagNames.push(tagName);
        }
        /* 标签属性处理 */
        var attrs = node.attrs;
        if(attrs){
            for(var j=0,len=attrs.length;j<len;j++){
                var attr = attrs[j];
                var attrName = attr.name;
                if(attrName.startsWith("@")||attrName.startsWith("v-on")){
                    var val = attr.value;
                    if(val.indexOf("$emit") != -1){
                        /* 解析emit */
                        var event = this.parseDomEventCaller(val);
                        if(event){
                            this.addEvent(event);
                        }
                    }
                }
            }
        }
        var children = node.childNodes;
        if(children){
            for(var i=0,l=children.length;i<l;i++){
                var child = children[i];
                this.analyzeDomNode(child);
            }
        }
    },

    /**
     * 解析emit字符串
     * @param {String} val emit字符串
    */
    parseDomEventCaller : function(val){
        try{
            var ast = recast.parse(val);
            var events = [];
            recast.visit(ast,{
                events : events,
                visitExpressionStatement: function(path) {
                    var node = path.value;
                    var exp = node.expression;
                    var callee = exp.callee;
                    if(callee){
                        if(callee.name=="$emit"){
                            var args = exp.arguments;
                            if(args.length>0){
                                var eventCode = args[0].value;
                                this.visitor.events.push({
                                    eventCode:eventCode,
                                    argumentNum : args.length-1
                                });
                            }else{
                                throw Error("解析事件参数错误，没有配置事件名称！");
                            }
                        }
                    }
                    this.traverse(path);
                }
            });
            if(events.length>0){
                return events[0];
            }else{
                return null;
            }
        }catch(e){
            throw new Error("解析事件参数错误，解析的字符串：[" + val + "]");
        }
    },

    analyzeJavascript : function(){
        if(!this.javascript) return;
        var ast = recast.parse(this.javascript);
        var events = [];
        recast.visit(ast,{
            events : events,
            visitExpressionStatement: function(path) {
                var node = path.value;
                var exp = node.expression;
                var callee = exp.callee;
                if(callee){
                    var obj = callee.object;
                    var property = callee.property;
                    if(obj&&property&&obj.type=="ThisExpression"&&property.type=="Identifier"&&property.name=="$emit"){
                        var args = exp.arguments;
                        var eventCode = args[0].value;
                        this.visitor.events.push({
                            eventCode:eventCode,
                            argumentNum : args.length-1
                        });
                    }
                }
                this.traverse(path);
            }
        });
        if(events && events.length > 0){
            for(var i = 0,l = events.length;i<l;i++){
                var event = events[i];
                this.addEvent(event);
            }
        }
    }

};

module.exports = VueComponentAnalyze;