const PluginTemplateGenerator = require('./PluginTemplateGenerator');
const path = require('path');
const fileUtils = require('../utils/FileUtils');
const componentUtils = require('../utils/ComponentUtils');
const juicer = require('juicer');
const beautify = require('js-beautify');
const ioUtils = require('../utils/IOUtils');
const fs = require('fs');
var dirName = __dirname;

var WindowTempateGenerator = function(params){
    PluginTemplateGenerator.call(this, params);
}
WindowTempateGenerator.prototype = PluginTemplateGenerator.prototype;

var new_prototype = {
    /**
     * 根据模板创建内容
     * @param {String} tplPath 模板路径
     * @param {Object} data 数据
     */
    createFile : function(tplPath, savePath, components) {
        return new Promise(function(resolve, reject){
            fs.readFile(tplPath, function(err, content){
                if (err) {
                    reject(err);
                    return;
                }
                juicer.set({
                    'strip': false,
                    'cache': false
                });
                var params = {
                    components: components
                };
                content = juicer(String(content), params);
                var subfix = savePath.substring(savePath.lastIndexOf('.') + 1);
                switch (subfix) {
                    case 'js':
                        content = beautify.js(content);
                        break;
                    case 'html':
                        content = beautify.html(content);
                        break;
                    case 'css':
                    case 'less':
                        content = beautify.css(content);
                        break;
                }
                var promise = ioUtils.write(savePath, content);
                promise.then(function() {
                    resolve();
                }).catch(function(e) {
                    reject(e);
                });
            });
        });
    },
    getTplRootPath : function() {
        /* 优先使用定义的来源目录 */
        var sourceDir = this.getSourceDir();
        if(sourceDir){
            var tplDir = path.resolve(sourceDir,"node_modules/v3webpack-cli/src/generator/template");
			if(fs.existsSync(tplDir)){
				return tplDir;
			}
        }
        return path.resolve(dirName, "template");
    },

    toPluginPath : function(tplPath) {
        var tplRootPath = this.getTplRootPath();
        var relativePath = tplPath.substring(tplRootPath.length + 1);
        if (relativePath.endsWith(".tpl")) {
            relativePath = relativePath.substring(0, relativePath.length - 4);
        }
        var baseDir = this.getBaseDir();
        return path.resolve(baseDir, relativePath);
    },

    dealPackageJson : function() {
        var packageJson = this.getPackageJson();
        var devDependencies = packageJson.devDependencies || {};
        devDependencies.autoprefixer = "^8.3.0";
        devDependencies["babel-core"] = "^6.26.0";
        devDependencies["babel-loader"] = "^7.1.4";
        devDependencies["babel-preset-es2015"] = "^6.24.1";
        devDependencies["babel-preset-stage-3"] = "^6.24.1";
        devDependencies["copy-webpack-plugin"] = "^4.5.1";
        devDependencies["css-loader"] = "^0.28.11";
        devDependencies["extract-text-webpack-plugin"] = "^3.0.2";
        devDependencies["global-dirs"] = "^0.1.1";
        devDependencies["html-webpack-plugin"] = "^3.2.0";
        devDependencies.less = "^3.0.1";
        devDependencies["cross-env"] = "^5.0.5";
        devDependencies["less-loader"] = "^4.1.0";
        devDependencies["postcss-url"] = "^7.3.2";
        devDependencies["v3-file-loader"] = "^1.0.0";
        devDependencies["vue-loader"] = "^14.2.2";
        devDependencies["vue-template-compiler"] = "^2.5.16";
        devDependencies.webpack = "^3.11.0";
        devDependencies["webpack-merge"] = "^4.1.2";
        this.pacakgeJson.devDependencies = devDependencies;
        var scripts = packageJson.scripts||{};
        scripts.dev = "cross-env NODE_ENV=development webpack-dev-server --config build/webpack.config.js";
        scripts.build = "cross-env NODE_ENV=production webpack --config build/webpack.config.js --progress --hide-modules";
        scripts["v3:package"] = "npm run build";
        scripts["v3:start"] = "npm run dev";
        packageJson.scripts = scripts;
        return ioUtils.write(packageJsonPath, JSON.stringify(packageJson, null, '\t'));
    },

    generate : function() {
        var components = componentUtils.analyzeComponents(this.getSourceDir());
        var tempPath = this.getTplRootPath();
        var files = fileUtils.listFiles(tempPath, true);
        var promises = [];
        var _this = this;
        files.forEach(function(tplPath) {
            var savePath = _this.toPluginPath(tplPath);
            if (!fs.existsSync(savePath)) {
                var promise = _this.createFile(tplPath, savePath, components);
                promises.push(promise);
            }else{
                console.warn('当前构件已存在该文件，创建被忽略！路径：' + savePath);
            }
        });
        // let promise = this.dealPackageJson();
        // promises.push(promise);
        return new Promise(function(resolve, reject) {
            Promise.all(promises).then(function()  {
                resolve();
            }).catch(function(err)  {
                reject(err);
            });
        });
    }
}
for(var key in new_prototype){
    if(new_prototype.hasOwnProperty(key)){
        WindowTempateGenerator.prototype[key] = new_prototype[key];
    }
}
module.exports = WindowTempateGenerator;