const globalDirs = require('global-dirs');
const fs = require('fs');
const path = require('path');

module.exports = {

    toPluginPath: function (pluginName, relativePath) {
        let selfPluginPath = path.resolve(__dirname, "../../node_modules", pluginName);
        if (fs.existsSync(selfPluginPath)) {
            return path.resolve(selfPluginPath, relativePath);
        } else {
            let globalPluginPath = path.resolve(globalDirs.npm.packages, pluginName);
            if (fs.existsSync(globalPluginPath)) {
                return path.resolve(globalPluginPath, relativePath);
            } else {
                throw new Error('当前环境未安装【' + pluginName + '】，请检查！');
            }
        }
    },

    postCssUrlCfg: {
        url: function (asset) {
            var res = asset.url;
            if (res) {
                res = res.trim();
                var prefix = res.indexOf("?") != -1 ? '&' : '?';
                res = res + prefix + "cssRes=true";
            }
            return res;
        }
    },

    getExternals: function () {
        let externals = {
            "vue": "Vue",
            "jQuery": "jQuery"
        };
        let modulePath = path.resolve(__dirname, '../../', 'node_modules');
        if (fs.existsSync(modulePath)) {
            let fileNames = fs.readdirSync(modulePath);
            if (fileNames && fileNames.length > 0) {
                fileNames.forEach(fileName => {
                    let absPath = path.resolve(modulePath, fileName, '.v3devrc');
                    if (fs.existsSync(absPath)) {
                        let packageJsonPath = path.resolve(modulePath, fileName, 'package.json');
                        let content = fs.readFileSync(packageJsonPath);
                        let cfg = JSON.parse(content);
                        externals[cfg.name] = cfg.name;
                    }
                });
            }
        }
        return externals;
    }

};