

var fs = require('fs');
var pathUtils = require("./PathUtils");
var vueTempCompiler = require("vue-template-compiler");
var VueComponentAnalyze = require("../generator/VueComponentAnalyze");
var ioUtils = require("./IOUtils");

module.exports = {
    /**
     * 分析div里面的标签和事件信息
     * @param {div对应的vue绝对路径} divFilePath 
     */
    analyzeDivContent : function(divFilePath){
        var absPath = divFilePath;
        /* 输出的信息 */
        var finishInfo = {
            $v3Platform$:{
                success:true,
                type:"analyzeSuccess",
                title:"JGDiv源码分析成功",
                content:""
            }
        }
        var content = "";
        try{
            /* 判断是否文件 */
            if(fs.statSync(absPath).isFile()){
                var absPath = divFilePath.replace(/\\/g, "/");
                var pathArr = absPath.split("/");
                var componentCode = pathArr[pathArr.length - 5];
                var windowCode = pathArr[pathArr.length - 3];
                var widgetCode = pathArr[pathArr.length - 2];
                fs.readFile(absPath, 'utf8', (function(cmCode,wnCode,wdCode){
                    return function(err,data){
                        var rs = vueTempCompiler.parseComponent(data);
                        var javascript = "";
                        var domScript = "";
                        if(rs.script){
                            javascript = rs.script.content;
                        }
                        if(rs.template){
                            domScript = rs.template.content;
                        }
                        var analyzer = new VueComponentAnalyze({
                            domScript : domScript,
                            javascript : javascript
                        });
                        var savePath = pathUtils.toWidgetMetadataPath(cmCode,wnCode,wdCode);
                        var metadata = {
                            events : analyzer.getEvents(),
                            tagnames : analyzer.getTagNames()
                        };
                        ioUtils.writeContent(pathUtils.getPluginPath()+"/dist/"+savePath,JSON.stringify(metadata,null,"\t"));
                    };
                })(componentCode,windowCode,widgetCode));
            }else{
                content = "非法文件路径，请检查. 文件路径：["+divFilePath+"]";
            }
        }catch(err){
            content = "分析出错, 错误信息：" + err;
        }
        if(content != ""){
            finishInfo.$v3Platform$.success = false;
            finishInfo.$v3Platform$.content = content;
        }
        console.log(JSON.stringify(finishInfo));
    }
}