var fs = require("fs");
var path = require("path");
var pathUtils = require("../utils/PathUtils");
var Component = require("../model/Component");
var Window = require("../model/Window");
var Widget = require("../model/Widget");
var platformComponentDir = "src/vendor/vplatform/components";
var componentSourceFolder = "src/components";
var componentSourcePath = path.join("./",componentSourceFolder);
var relativePath = path.join("../",platformComponentDir);
var webpackRelativePath = path.join("./",platformComponentDir);
var metadataPath = "src/components/metadata.json";

module.exports = {

  /**
   * 分析构件，获取构件、窗体信息
   */
  analyzeComponents : function(tplPath){
    var pluginPath = tplPath ? tplPath : pathUtils.getPluginPath();
    var metdataAbsPath = path.join(pluginPath,metadataPath);
    try{
      var results = [];
      var metdataStr = fs.readFileSync(metdataAbsPath);
      var metadata = JSON.parse(metdataStr);
      for(var i=0,l=metadata.length;i<l;i++){
        var cmp = metadata[i];
        var componentCode = cmp.code;
        var windows = [];
        var wins = cmp.windows;
        if(wins){
          for(var j=0,len=wins.length;j<len;j++){
            var win = wins[j];
            var windowCode = win.code;
            var widgets = [];
            var wds = win.widgets;
            if(wds){
              for(var k=0,ll=wds.length;k<ll;k++){
                var wd = wds[k];
                widgets.push(new Widget({
                  code : wd.code,
                  previewIndexPath : "./"+pathUtils.getWidgetPreviewIndexPath(componentCode,windowCode,wd.code)
                }));
              }
            }
            windows.push(new Window({
              code : windowCode,
              name : win.name,
              widgets : widgets,
              indexPath : "./"+pathUtils.getWindowIndexPath(componentCode,windowCode),
              previewIndexPath : "./"+pathUtils.getWindowPreviewIndexPath(componentCode,windowCode)
            }));
          }
        }
        results.push(new Component({
          code : componentCode,
          name : cmp.name,
          srcPath : pathUtils.getComponentSrcPath(componentCode),
          windows : windows
        }));
      }
      return results;
    }catch(e){
      throw Error("解析json文件失败，请检查文件内容！错误原因："+e.message+"，文件路径："+metadataPath);
    }  
  }

};
