const http = require('http');
const commandFactory = require('./commander/CommandFactory');
const CommandContext = require('./commander/CommandContext');
const CommandContextKey = require('./commander/CommandContextKey');
const CommandResult = require('./commander/CommandResult');
const querystring = require('querystring');

class HttpServer {

    constructor(port) {
        this.port = port;
        this.context = new CommandContext();
        this.context.put(CommandContextKey.HTTP_SERVER, this);
    }

    start() {
        this.server = http.createServer((request, reponse) => {
            let body = [],start=Date.now();
            request.on('data', (chunk) => {
                body.push(chunk);
            });
            request.on('end', () => {
                try {
                    body = Buffer.concat(body);
                    let params = querystring.parse(body.toString());
                    let command = params.commander;
                    let commander = commandFactory.getCommand(command);
                    if (commander) {
                        let promise = commander.invoke(params, this.context);
                        promise.then(result => {
                            reponse.writeHead(200, {
                                'Content-Type': 'application/json'
                            });
                            this.log(start);
                            reponse.end(JSON.stringify(result.toJson()));
                        }).catch(err => {
                            reponse.writeHead(200, {
                                'Content-Type': 'application/json'
                            });
                            let result = new CommandResult({
                                success: false,
                                type: "CommanderInvokeException",
                                title: `Commander[${command}]执行有误`,
                                content: typeof (err) == "string" ? err : (err.stack ? err.stack : err.message)
                            });
                            this.log(start);
                            reponse.end(JSON.stringify(result.toJson()));
                        });
                    } else {
                        reponse.writeHead(200, {
                            'Content-Type': 'application/json'
                        });
                        let result = new CommandResult({
                            success: false,
                            type: "CommanderException",
                            title: "请求commander参数传递有误",
                            content: ""
                        });
                        this.log(start);
                        reponse.end(JSON.stringify(result.toJson()));
                    }
                } catch (err) {
                    reponse.writeHead(200, {
                        'Content-Type': 'application/json'
                    });
                    let result = new CommandResult({
                        success: false,
                        type: "SystemError",
                        title: "系统内部执行错误",
                        content: typeof (err) == "string" ? err : (err.stack ? err.stack : err.message)
                    });
                    this.log(start);
                    reponse.end(JSON.stringify(result.toJson()));
                }
            });

        }).listen(this.port);
    }

    close() {
        return new Promise((resolve, reject) => {
            let commands = commandFactory.getCommands();
            let promises = [];
            if (commands && commands.length > 0) {
                commands.forEach(command => {
                    let promise = command.beforeClose(this.context);
                    promises.push(promise);
                });
            }
            Promise.all(promises).then(() => {
                if (this.server) {
                    this.server.close();
                }
                resolve();
            }).catch(err => {
                reject(err);
            });
        });
    }

    log(start){
        console.log(`已完成前端请求响应，总耗时【${Date.now()-start}】毫秒！\n`);
    }

}

module.exports = HttpServer;