const http = require('http');
const ioUtils = require('../utils/IOUtils');
const path = require('path');
const HttpServer = require('./HttpServer');

module.exports = {

    _detectPort: function (port) {
        return new Promise((resolve, reject) => {
            let server = http.createServer().listen(port);
            let calledOnce = false;
            let timeoutRef = setTimeout(() => {
                calledOnce = true;
                resolve({
                    usable: false,
                    port: port
                });
            }, 2000);
            timeoutRef.unref();
            let connected = false;
            server.on('listening', () => {
                clearTimeout(timeoutRef);
                if (server)
                    server.close();
                if (!calledOnce) {
                    calledOnce = true;
                    resolve({
                        usable: true,
                        port: port
                    });
                }
            });
            server.on('error', err => {
                clearTimeout(timeoutRef);
                let result = true;
                if (err.code === 'EADDRINUSE')
                    result = false;
                if (!calledOnce) {
                    calledOnce = true;
                    resolve({
                        usable: result,
                        port: port
                    });
                }
            });
        });
    },

    _getValidPort: function () {
        let func = (port)=>{
            return new Promise((resolve,reject)=>{
                let promise = this._detectPort(port);
                promise.then(data=>{
                    if(data.usable){
                        resolve(data.port);
                    }else{
                        let p = func(port+1);
                        p.then(pt=>{
                            resolve(pt);
                        });
                    }
                });
            });
        };
        return new Promise((resolve, reject) => {
            let metadata = require('../../metadata.json');
            let port = metadata.port;
            let promise = func(port);
            promise.then(pt => {
                if(pt!=port){
                    metadata.port = pt;
                    let pm = ioUtils.write(path.resolve(),JSON.stringify(metadata,null,"\t"));
                    pm.then(()=>{
                        resolve(pt);
                    }).catch(err=>{
                        reject(err);
                    });
                }
                resolve(pt);
            });
        });
    },

    start: function () {
        return new Promise((resolve,reject)=>{
            let promise = this._getValidPort();
            promise.then(port=>{
                let server = new HttpServer(port);
                server.start();
                resolve();
            }).catch(err=>{
                reject(err);
            });
        });
    }

};