const path = require('path');
const fs = require('fs');
const ioUtils = require('../utils/IOUtils');
const fileUtils = require('../utils/FileUtils');
const versionUtils = require('../utils/VersionUtils');

class NodejsPluginUpdater{

    constructor(src){
        this.src = src;
    }

    update(){
        return new Promise((resolve,reject)=>{
            let start = Date.now();
            fs.readdir(path.resolve(this.src,"node_modules"),(err,filenames)=>{
                if(err){
                    reject(err);
                    return;
                }
                if(filenames&&filenames.length>0){
                    let promises = [];
                    filenames.forEach(filename => {
                        let name = path.basename(filename);
                        let srcPath = path.resolve(this.src,filename);
                        let distPath = path.resolve(__dirname,"../../node_modules",filename);
                        let promise;
                        if(name==".bin"){
                            promise = ioUtils.copyDirWithFilter(srcPath,distPath,(filePath)=>{
                                let absPath = path.resolve(distPath,path.relative(srcPath,filePath));
                                return !fileUtils.exists(absPath);
                            });
                        }else{
                            let selfPackageJsonPath = path.resolve(distPath,"package.json");
                            let needUpdate = false,packageJson;
                            if(fileUtils.exists(selfPackageJsonPath)){
                                packageJson = require(path.resolve(srcPath,"package.json"));
                                try{
                                    let selfPackageJson = require(selfPackageJsonPath);
                                    needUpdate = versionUtils.isNew(packageJson.version,selfPackageJson.version);
                                }catch(e){
                                    needUpdate = true;
                                }                                
                            }else{
                                needUpdate = true;
                            }
                            if(needUpdate){
                                console.log('nodejs插件需要更新，插件名称：'+filename);
                                let promise1 = ioUtils.copyDirWithFilter(srcPath,distPath,(absPath,distPath)=>{
                                    delete require.cache[distPath];
                                    return true;
                                });
                                promise = new Promise((resolve,reject)=>{
                                    promise1.then(()=>{
                                        //更新缓存
                                        delete require.cache[selfPackageJsonPath];
                                        resolve();
                                    }).catch(err=>{
                                        reject(err);
                                    });
                                });
                            }
                        }
                        promises.push(promise);
                    });
                    Promise.all(promises).then(()=>{
                        //TODO 检查构件package.json中是否有未安装的构件
                        console.log(`更新构件Nodejs插件完成，耗时【${Date.now()-start}】毫秒！`);
                        resolve();
                    }).catch(err=>{
                        reject(err);
                    });
                }else{
                    resolve();
                }
            });
        });
    }

}

module.exports = NodejsPluginUpdater;