const fs = require('fs');
const path = require('path');
const pathUtils = require('./PathUtils');

module.exports = {

	makeDir: function (path) {
		let iter = (parent) => {
			if (!this.exists(parent)) {
				let f = pathUtils.getParent(parent);
				iter(f);
				fs.mkdirSync(parent);
			}
		};
		let parent = pathUtils.getParent(path);
		iter(parent);
		if (!this.exists(path)) {
			fs.mkdirSync(path);
		}
	},

	exists: function (path) {
		return fs.existsSync(path);
	},

	isFile: function (path) {
		if (fs.existsSync(path)) {
			let stat = fs.statSync(path);
			return stat.isFile();
		}
		return false;
	},

	isDirectory: function (path) {
		if (fs.existsSync(path)) {
			let stat = fs.statSync(path);
			return stat.isDirectory();
		}
		return false;
	},

	listFiles: function (filePath, recursion) {
		var files = [];
		if (fs.existsSync(filePath)) {
			var stat = fs.statSync(filePath);
			if (stat.isFile()) {
				files.push(filePath);
			} else if (stat.isDirectory() && recursion) {
				var paths = fs.readdirSync(filePath);
				for (var i = 0, len = paths.length; i < len; i++) {
					files = files.concat(this.listFiles(path.resolve(filePath, paths[i]), recursion));
				}
			}
		}
		return files;
	}

};