const fileUtils = require('./FileUtils');
const pathUtils = require('./PathUtils');
const fs = require('fs');
const path = require('path');

class CopyFileQueue{
    constructor(limit){
        this.limit = limit;
        this.waitingQueue = [];
        this.runningCount = 0;
    }
    push(src,dist){
        return new Promise((resolve,reject)=>{
            this.waitingQueue.push({
                src:src,
                dist:dist,
                resolve:resolve,
                reject:reject
            });
            this.start();
        });
    }
    start(){
        if(this.runningCount<this.limit){
            let offLen = this.limit-this.runningCount;
            let taskList;
            if(this.waitingQueue.length<offLen){
                taskList = this.waitingQueue;
                this.waitingQueue = [];
            }else{
                taskList = this.waitingQueue.splice(0,offLen);
            }
            this.runningCount += taskList.length;
            taskList.forEach(params=>{
                this.run(params);
            });
        }
    }
    run(params){
        fileUtils.makeDir(pathUtils.getParent(params.dist));
        let input = fs.createReadStream(params.src);
        let output = fs.createWriteStream(params.dist);
        input.pipe(output);
        input.on('close', () => {
            this.runningCount--;
            this.start();
            params.resolve();
        });
        input.on('error', (err) => {
            params.reject(err);
        }); 
    }
}

module.exports = {

    copyQueue : new CopyFileQueue(2000),

    copyFile: function (src, dist) {
        return this.copyQueue.push(src,dist);
    },

    copyDirWithFilter: function (src, dist, filter) {
        return new Promise((resolve, reject) => {
            if (!fileUtils.exists(src)) {
                reject(new Error("拷贝目录失败！原因：源目录不存在，path=" + src));
            }
            
            if (fileUtils.isDirectory(src)) {
                fs.readdir(src, (err, fileNames) => {
                    if (err) {
                        reject(err);
                        return;
                    }
                    if (fileNames && fileNames.length > 0) {
                        let promises = [];
                        fileNames.forEach(filename => {
                            let absPath = path.resolve(src, filename);
                            let promise;
                            if (fileUtils.isFile(absPath)) {
                                let distPath = path.resolve(dist, filename);
                                let flag = filter ? filter(absPath,distPath) : true;
                                if (flag) {
                                    promise = this.copyFile(absPath, distPath);
                                }
                            } else if (fileUtils.isDirectory(absPath)) {
                                promise = this.copyDirWithFilter(absPath, path.resolve(dist, filename), filter);
                            }
                            if (promise) {
                                promises.push(promise);
                            }
                        });
                        Promise.all(promises).then(() => {
                            resolve();
                        }).catch(err => {
                            reject(err);
                        });
                    } else {
                        resolve();
                    }
                });
            } else {
                reject(new Error("拷贝目录失败！原因：路径错误，不是文件夹，path=" + src));
            }
        });
    },

    write : function(filePath,content){
        fileUtils.makeDir(pathUtils.getParent(filePath));
        return new Promise((resolve, reject) => {
            fs.writeFile(filePath, content, (err) => {
                if (err) {
                    reject(err);
                    return;
                }
                resolve();
            });
        });
    }

};